/* This file is a part of the SimpleScalar tool suite written by
 * Todd M. Austin, University of Wisconsin - Madison, Computer Sciences
 * Department as a part of the Multiscalar Research Project.
 *  
 * The SimpleScalar x86 port was written by Steve Bennett.
 * 
 * The tool suite is currently maintained by Doug Burger.
 * 
 * Copyright (C) 1994, 1995, 1996 by Todd M. Austin
 *
 * This source file is distributed "as is" in the hope that it will be
 * useful.  The tool set comes with no warranty, and no author or
 * distributor accepts any responsibility for the consequences of its
 * use. 
 * 
 * Everyone is granted permission to copy, modify and redistribute
 * this tool set under the following conditions:
 * 
 *    This source code is distributed for non-commercial use only. 
 *    Please contact the maintainer for restrictions applying to 
 *    commercial use.
 *
 *    Permission is granted to anyone to make or distribute copies
 *    of this source code, either as received or modified, in any
 *    medium, provided that all copyright notices, permission and
 *    nonwarranty notices are preserved, and that the distributor
 *    grants the recipient permission for further redistribution as
 *    permitted by this document.
 *
 *    Permission is granted to distribute this file in compiled
 *    or executable form under the same conditions that apply for
 *    source code, provided that either:
 *
 *    A. it is accompanied by the corresponding machine-readable
 *       source code,
 *    B. it is accompanied by a written offer, with no time limit,
 *       to give anyone a machine-readable copy of the corresponding
 *       source code in return for reimbursement of the cost of
 *       distribution.  This written offer must permit verbatim
 *       duplication by anyone, or
 *    C. it is distributed by someone who received only the
 *       executable form, and is accompanied by a copy of the
 *       written offer of source code that they received concurrently.
 *
 * In other words, you are welcome to use, share and improve this
 * source file.  You are forbidden to forbid anyone else to use, share
 * and improve what you give them.
 *
 * INTERNET: dburger@cs.wisc.edu
 * US Mail:  1210 W. Dayton Street, Madison, WI 53706
 */

/***********************************************************************
    INCLUDES
 ***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <setjmp.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/time.h>
#ifndef sparc
#include <sys/ioctl.h>
#else
#include <sys/ttold.h>
#endif
#include <termio.h>
#include <termios.h>

/*
#include "ix86_state.h"
#include "errorRoutines.h"
*/
#include "misc.h"

/* prototypes for the various simulators */
#include "func.h"


/* missing syscall decl */
extern int ioctl(int fd, int req, caddr_t arg);

enum sim_type {
	SimFunc,
	SimSuperscalar,
	SimMultiscalar
};


/***********************************************************************
 * Forward Declarations
 ***********************************************************************/
static void setupSignalHandlers(void);
static void DumpStats(int sigtype);
static void parseGlobalOptions(int argc, char **argv, char **envp);



/***********************************************************************
 * MAIN()
 ***********************************************************************/
void
main(int argc, char **argv, char **envp)
{
	int            exitcode;
	struct termios termstruct;
	enum sim_type  simType;

	/* register signal handlers with OS */
	setupSignalHandlers();

	/* only one simulator is implemented to date. */
	simType = SimFunc;

	/*
	 * save the current terminal info so that we can recover if we
	 * exit early and the term stuff has been changed by the
	 * target program (specifically, sc does this)
	 */
	ioctl(2, TCGETS, (caddr_t)&termstruct);

	/* set up a non-local exit point */
	if ((exitcode = setjmp(exitBuf)) != 0) {
		exitcode--; /* special handling as longjmp cannot pass 0 */

		/* Recover the old terminal characteristics */
		ioctl(2, TCSETS, (caddr_t)&termstruct);

		DumpFinalStats(stderr);
		exit(exitcode);
	}

	/*
	 * Parse command line options
	 * In 2 steps...
	 */

	/* first, options that apply to all simulators */
	parseGlobalOptions(argc, argv, envp);

	/* next, options that apply to specific simulators */
	switch(simType) {
	    case SimFunc:
		SimOptions(argc, argv);
			break;

	    case SimSuperscalar:
	    case SimMultiscalar:
	    default:
		error("bad simulator type");
		exit(-1);
		break;
	}

	/* make sure all command line arguments have been consumed */
	if (optInd == argc) {
		Usage(argc, argv);
		exit(1);
	}


	/* configure the specific simulator */
	switch(simType) {
	    case SimFunc:
		SimConfig();
		break;

	    case SimSuperscalar:
	    case SimMultiscalar:
	    default:
		error("bad simulator type");
		exit(-1);
		break;
	}

	/* record start of execution time, used in rate stats */
	startTime = time((time_t *)NULL);

	/* call the specific simulator */
	switch(simType) {
	    case SimFunc:
		SimMain(argc-optInd, argv+optInd, envp);
		break;

	    case SimSuperscalar:
	    case SimMultiscalar:
	    default:
		error("bad simulator type");
		exit(-1);
		break;
	}

	/* the simulator should not return to main. */
	error("returned to main");

}/*** end main() ***/




/***********************************************************************
 * DUMPSTATS(), the signal handler
 ***********************************************************************/
static
void
DumpStats(int sigtype)
{
	dumpStats = TRUE;

}/*** end DumpStats() ***/



/***********************************************************************
 * setupSignalHandlers()
 ***********************************************************************/
static
void
setupSignalHandlers(void)
{

#if !defined(CONDOR)
	signal(SIGQUIT, HandleSIGQUIT);
	signal(SIGILL,  HandleSIGILL);
	signal(SIGABRT, HandleSIGABRT);
	signal(SIGEMT,  HandleSIGEMT);
	signal(SIGFPE,  HandleSIGFPE);
	signal(SIGBUS,  HandleSIGBUS);
	signal(SIGSEGV, HandleSIGSEGV);
	signal(SIGSYS,  HandleSIGSYS);
	signal(SIGLOST, HandleSIGLOST);
#endif
	/* catch SIGUSR1 and dump intermediate stats */
	signal(SIGUSR1, DumpStats);

}/*** end setupSignalHandlers() ***/




static
void
parseGlobalOptions(int argc, char **argv, char **envp)
{
	char     c;
	int      i;

	/* print out the program arguments */
	fprintf(stderr, "**ARGS**: ");
	for (i = 0; i < argc; i++)
		fprintf(stderr, "%s ", argv[i]);
	fprintf(stderr, "\n");

	if (argc < 2) {
		Usage(argc, argv);
		exit(1);
	}

	/* The only options be processed globally are verbosity switches */
	verbose        = FALSE;
	syscallVerbose = FALSE;

	NewOpt();
	while ((c = GetOpt(argc, argv, ALL_OPTIONS)) != EOF) {
		switch (c) {
		case 'V':
			syscallVerbose = TRUE;
			break;
		case 'v':
		{
			unsigned long i;
			char c;
			if (sscanf(optArg, "%c%lx", &c, &i) != 2)
				fatal("could not parse verbose parameters");
			if (c == 't') {
				verbTime = i;
				info("verbosity starting @time 0x%08x...", verbTime);
			} else if(c == 'i') {
				verbInst = i;
				info("verbosity starting @instruction number 0x%08x...", verbInst);
			} else {
				verbAddr = i;
				info("verbosity starting @address 0x%08x...", verbAddr);
			}
			if (verbInst == 1)
				verbose = TRUE;
		}
			break;
		case '?':
			Usage(argc, argv);
			exit(1);
		}
	}
	
}/*** parseGlobalOptions() ***/
